﻿using System;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.MainForm;
using HeuristicLab.Optimization;


namespace HeuristicLab.HiveJobUploader {
  public partial class HiveJobUploaderForm : Form {
    Semaphore step = new Semaphore(0, 1);

    public HiveJobUploaderForm() {
      InitializeComponent();
      ContentManager.Initialize(new PersistenceContentManager());
    }

    private void btnAdd_Click(object sender, EventArgs e) {
      if (openFileDialog1.ShowDialog() == DialogResult.OK) {
        foreach (string name in openFileDialog1.FileNames) {
          if (!lbxHiveTasks.Items.Contains(name)) {
            lbxHiveTasks.Items.Add(name);
          } 
        }
      }
    } 

    private void HandleEx(Exception ex) {
      MessageBox.Show(ex.ToString());
    }

    private void HiveJobUploaderForm_Load(object sender, EventArgs e) {
      uploadProgress.Visible = false;
      openFileDialog1.Multiselect = true;
      lblState.BackColor = Color.Transparent;
      toolTip1.SetToolTip(btnAdd, "Add Files");
      toolTip1.SetToolTip(btnUpload, "Upload the files added");
    }

    private void UpdatelblState(string message) {
      if (InvokeRequired) {
        lblState.Invoke(new Action<string>(UpdatelblState), message);
      } else
        lblState.Text = message;
    }

    private void UpdateProgressbar(int i) {
      if (InvokeRequired) {
        uploadProgress.Invoke(new Action<int>(UpdateProgressbar), i);
      } else {
        double progressValue = lbxHiveTasks.Items.Count;
        uploadProgress.Visible = true;
        uploadProgress.Minimum = 0;
        uploadProgress.Maximum = 100;
        uploadProgress.Value = (int)Math.Round(i /( progressValue) * 100);
        uploadProgress.Step = 1;
      }
    }

    private void UploadFile(string file) {
        IStorableContent content = ContentManager.Load(file);
        IOptimizer opt = (IOptimizer)content;
        if (opt == null)
          throw new Exception("File does not contain an IOptimizer!");
        HiveClient.Instance.Refresh();
        RefreshableJob refJob = new RefreshableJob();
        refJob.HiveTasks.Add(new OptimizerHiveTask(opt));
        HiveClient.StartJob(new Action<Exception>(HandleEx)
        , refJob, new System.Threading.CancellationToken());
        step.Release();
    }

    private void btnUpload_Click(object sender, EventArgs e) {
      if (InvokeRequired) Invoke((Action<object, EventArgs>)btnUpload_Click);
      else {
        int i = 1;
        uploadProgress.Visible = true;
        foreach (string item in lbxHiveTasks.Items) {
          Thread uploader = new Thread(() => UploadFile(item));
          uploader.IsBackground = true;
          uploader.Start();
          UpdatelblState( "Uploading " + item);
          UpdateProgressbar(i);
          step.WaitOne();
          i++;
        }
        UpdatelblState("Uploading Finished");
        uploadProgress.Visible = false;
        lbxHiveTasks.Items.Clear();
      }
    }

  }
}
